@@ -6,9 +6,20 @@
 #' @return
 #' vector contains Longitude and Latitude in numeric
 str_loc_to_num_coord <- function(str_location) {
-  if (is.na(str_location) || !nzchar(str_location)) return(c(NA_real_, NA_real_))
-  parts <- strsplit(str_location, split = ",", fixed = TRUE)[[1L]]
-  as.numeric(parts)
+  if (length(str_location) == 0) {
+    return(c(NA_real_, NA_real_))
+  }
+  empty <- is.na(str_location) | !nzchar(str_location)
+  out <- matrix(NA_real_, nrow = length(str_location), ncol = 2)
+  if (any(!empty)) {
+    parts <- strsplit(str_location[!empty], split = ",", fixed = TRUE)
+    coords <- lapply(parts, function(x) as.numeric(x[1:2]))
+    out[!empty, ] <- do.call(rbind, coords)
+  }
+  if (length(str_location) == 1L) {
+    return(out[1L, ])
+  }
+  out
 }
 
 #' Take longitude and latitude from location string out.
